# SPDX-FileCopyrightText: 2023-present Oori Data <info@oori.dev>
#
# SPDX-License-Identifier: Apache-2.0
# wordloom

# ruff: noqa: F401,F403

from .__about__ import __version__
from .wordloom import load, language_item, LI
